/*
 * fuzzystrmatch patch
 *     Alphagram based fuzzy string matching by OMNIconnect (www.omniconnect.com.au)
 *
 * alphagram.c (Anagrams arranged alphabeticaly)
 *
 * More functions for "fuzzy" comparison of strings
 *
 * Ian Small <ians@omniconnect.com.au>
 *
 * Copyright (c) 2008-2009, OMNIconnect
 * ALL RIGHTS RESERVED;
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHOR OR DISTRIBUTORS HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHOR AND DISTRIBUTORS HAS NO OBLIGATIONS TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 */

 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <executor/spi.h>
 #include <fmgr.h>
 #include <ctype.h>

 PG_MODULE_MAGIC;

 #define GET_STR(textp) DatumGetCString(DirectFunctionCall1(textout, PointerGetDatum(textp)))
 #define GET_TEXT(cstrp) DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(cstrp)))

 extern Datum alphagram(PG_FUNCTION_ARGS); // Perfect alpha numeric alphagram
 extern Datum alphagram_alpha(PG_FUNCTION_ARGS); // Perfect alpha alphagram
 extern Datum alphagram_numeric(PG_FUNCTION_ARGS); // Perfect numeric alphagram
 extern Datum imperfect_alphagram(PG_FUNCTION_ARGS); // Imperfect alpha numeric alphagram
 extern Datum imperfect_alphagram_alpha(PG_FUNCTION_ARGS); // Imperfect alpha alphagram
 extern Datum imperfect_alphagram_numeric(PG_FUNCTION_ARGS); // Imperfect numeric alphagram

 PG_FUNCTION_INFO_V1(alphagram);
 Datum alphagram(PG_FUNCTION_ARGS)
 {
   char *s = GET_STR(PG_GETARG_TEXT_P(0));
   int i=0, n, p=0;
   int a[44];
   bzero(a, sizeof(a));

   for(n=0; n<strlen(s); n++)
   {
     if(isalnum(s[n]))             // ONLY Alpha Numeric NO Spaces
       a[(int)toupper(s[n])-48]++; // Count how many of this char
   }
   for(n=0; n<10; n++) // Numeric
   {
     for(i=0; i<a[n]; i++) // How Many?
     {
       s[p] = n+48; // Put the char in next s position
       p++;
     }
   }
   for(n=17; n<43; n++) // Alpha
   {
     for(i=0; i<a[n]; i++) // How Many?
     {
       s[p] = n+48; // Put the char in next s position
       p++;
     }
   }
   s[p] = 0;
   PG_RETURN_TEXT_P( DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(s))) );
 }

 PG_FUNCTION_INFO_V1(alphagram_alpha);

 Datum alphagram_alpha(PG_FUNCTION_ARGS) // Perfect alpha only alphagram
 {
   char *s = GET_STR(PG_GETARG_TEXT_P(0));
   int i=0, n, p=0;
   int a[44];
   bzero(a, sizeof(a));

   for(n=0; n<strlen(s); n++)
   {
     if(isalnum(s[n])) a[(int)toupper(s[n])-48]++;
   }
   for(n=17; n<43; n++) // Alpha Only
   {
     for(i=0; i<a[n]; i++)
     {
       s[p] = n+48;
       p++;
     }
   }
   s[p] = 0;
   PG_RETURN_TEXT_P( DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(s))) );
 }

 PG_FUNCTION_INFO_V1(alphagram_numeric);

 Datum alphagram_numeric(PG_FUNCTION_ARGS) // Perfect numeric only alphagram
 {
   char *s = GET_STR(PG_GETARG_TEXT_P(0));
   int i=0, n, p=0;
   int a[44];
   bzero(a, sizeof(a));

   for(n=0; n<strlen(s); n++)
   {
     if(isalnum(s[n])) a[(int)toupper(s[n])-48]++;
   }
   for(n=0; n<10; n++) // Numeric Only
   {
     for(i=0; i<a[n]; i++)
     {
       s[p] = n+48;
       p++;
     }
   }
   s[p] = 0;
   PG_RETURN_TEXT_P( DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(s))) );
 }

 PG_FUNCTION_INFO_V1(imperfect_alphagram);

 Datum imperfect_alphagram(PG_FUNCTION_ARGS) // Imperfect alpha numeric alphagram
 {
   char *s = GET_STR(PG_GETARG_TEXT_P(0));
   int n, p=0;
   int a[44];
   bzero(a, sizeof(a));

   for(n=0; n<strlen(s); n++)
     if(isalnum(s[n]))
       a[(int)toupper(s[n])-48] = toupper(s[n]);

   for(n=0; n<10; n++) // Numeric
     if(a[n])
     {
       s[p] = a[n];
       p++;
     }

   for(n=17; n<43; n++) // Alpha
     if(a[n])
     {
       s[p] = a[n];
       p++;
     }
   s[p] = 0;
   PG_RETURN_TEXT_P( DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(s))) );
 }

 PG_FUNCTION_INFO_V1(imperfect_alphagram_alpha);

 Datum imperfect_alphagram_alpha(PG_FUNCTION_ARGS) // Imperfect alpha only alphagram
 {
   char *s = GET_STR(PG_GETARG_TEXT_P(0));
   int n, p=0;
   int a[44];
   bzero(a, sizeof(a));

   for(n=0; n<strlen(s); n++)
     if(isalnum(s[n]))
       a[(int)toupper(s[n])-48] = toupper(s[n]);

   for(n=17; n<43; n++) // Alpha Only
     if(a[n])
     {
       s[p] = a[n];
       p++;
     }
   s[p] = 0;
   PG_RETURN_TEXT_P( DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(s))) );
 }

 PG_FUNCTION_INFO_V1(imperfect_alphagram_numeric);

 Datum imperfect_alphagram_numeric(PG_FUNCTION_ARGS) // Imperfect numeric only alphagram
 {
   char *s = GET_STR(PG_GETARG_TEXT_P(0));
   int n, p=0;
   int a[44];
   bzero(a, sizeof(a));

   for(n=0; n<strlen(s); n++)
     if(isalnum(s[n]))
       a[(int)toupper(s[n])-48] = toupper(s[n]);

   for(n=0; n<10; n++) // Numeric Only
     if(a[n])
     {
       s[p] = a[n];
       p++;
     }
   s[p] = 0;
   PG_RETURN_TEXT_P( DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(s))) );
 }

