
 --  EXAMPLES using the OMNIconnect alphagram functions

 --  fuzzystrmatch patch
 --      Alphagram based fuzzy string matching
 -- 
 --  alphagram.c (Anagrams arranged alphabeticaly)
 -- 
 --  More functions for "fuzzy" comparison of strings
 --  
 --  Ian Small <ians@omniconnect.com.au>
 --  
 --  OMNIconnect PTY LTD
 --  14 Elite Way
 --  Carrum Downs
 --  VICTORIA 3201
 --  AUSTRALIA
 --  
 --  www.omniconnect.com.au
 -- 
 --  Copyright (c) 2008-2009, OMNIconnect PTY LTD
 --  ALL RIGHTS RESERVED;
 --  
 --  Permission to use, copy, modify, and distribute this software and its
 --  documentation for any purpose, without fee, and without a written agreement
 --  is hereby granted, provided that the above copyright notice and this
 --  paragraph and the following two paragraphs appear in all copies.
 --  
 --  IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR
 --  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 --  LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 --  DOCUMENTATION, EVEN IF THE AUTHOR OR DISTRIBUTORS HAVE BEEN ADVISED OF THE
 --  POSSIBILITY OF SUCH DAMAGE.
 --  
 --  THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 --  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 --  AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 --  ON AN "AS IS" BASIS, AND THE AUTHOR AND DISTRIBUTORS HAS NO OBLIGATIONS TO
 --  PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.



 -- Example SQL query using the alphagram function
 -- This will list all localities that are anagrams of each other
 SELECT distinct a.locality, alphagram(a.locality)
 FROM postcode a, postcode b
 WHERE alphagram(a.locality) = alphagram(b.locality)
 AND a.locality <> b.locality
 ORDER BY alphagram(a.locality);

 -- RETURNS
 -- locality    alphagram
 -- NANTAWARRA  AAAANNRRTW
 -- TARRAWANNA  AAAANNRRTW
 -- LARAMBA     AAABLMR
 -- MALABAR     AAABLMR
 -- WANGARA     AAAGNRW
 -- WARANGA     AAAGNRW
 -- YALATA      AAALTY
 -- YATALA      AAALTY
 -- COMBARA     AABCMOR
 -- CORAMBA     AABCMOR
 -- BUNDARA     AABDNRU
 -- BURANDA     AABDNRU
 -- BAMGANIE    AABEGIMN
 -- NAGAMBIE    AABEGIMN
 -- etc.

*********************************************************************************

 -- Example SQL query using the imperfect_alphagram function
 -- This will list all localities that are imperfect anagrams of each other
 SELECT distinct a.locality, imperfect_alphagram(a.locality)
 FROM postcode a, postcode b
 WHERE imperfect_alphagram(a.locality) = imperfect_alphagram(b.locality)
 AND a.locality <> b.locality
 ORDER BY imperfect_alphagram(a.locality);

 -- RETURNS
 -- locality                                 imperfect_alphagram
 -- ROBINVALE IRRIGATION DISTRICT SECTION B  ABCDEGILNORSTV
 -- ROBINVALE IRRIGATION DISTRICT SECTION C  ABCDEGILNORSTV
 -- ROBINVALE IRRIGATION DISTRICT SECTION D  ABCDEGILNORSTV
 -- ROBINVALE IRRIGATION DISTRICT SECTION E  ABCDEGILNORSTV
 -- BANDY     CREEK                          ABCDEKNRY
 -- BRANDY    CREEK                          ABCDEKNRY
 -- BRENDALE BC                              ABCDELNR
 -- BRENDALE DC	                             ABCDELNR
 -- BRUCEDALE                                ABCDELRU
 -- CULLACABARDEE                            ABCDELRU
 -- BUNDALL BC                               ABCDLNU
 -- BUNDALL DC                               ABCDLNU
 -- FORREST BEACH                            ABCEFHORST
 -- FORRESTERS BEACH                         ABCEFHORST
 -- etc.

*********************************************************************************

 -- Example SQL query using the imperfect_alphagram function
 -- This correct a spelling mistake
 SELECT a.state, a.pcode, a.locality, b.search, imperfect_alphagram(b.search)
 FROM postcode a, (SELECT upper('wdigemlotha') as search) b
 WHERE imperfect_alphagram(a.locality) = imperfect_alphagram(b.search)
 ORDER BY a.locality;

 -- RETURNS
 -- state  pcode  locality        search        imperfect_alphagram
 -- WA     6443   WIDGIEMOOLTHA   WDIGEMLOTHA   ADEGHILMOTW
 -- 1 row(s)

*********************************************************************************

 -- Example SQL query using the imperfect_alphagram function and levenshtein Function
 SELECT a.state, a.pcode, a.locality, b.search, imperfect_alphagram(b.search), levenshtein(a.locality, b.search)
 FROM postcode a, (SELECT upper('bary') as search) b
 WHERE imperfect_alphagram(a.locality) = imperfect_alphagram(b.search)
 ORDER BY levenshtein(a.locality, b.search);

 -- RETURNS
 -- state pcode locality search  imperfect_alphagram  levenshtein
 -- NSW   2340  BARRY    BARY    ABRY                 1
 -- NSW   2799  BARRY    BARY    ABRY                 1
 -- NSW   2566  RABY     BARY    ABRY                 2
 -- SA    5276  BRAY     BARY    ABRY                 2
 -- NSW   2469  YABBRA   BARY    ABRY                 4
 -- NSW   2446  BYABARRA BARY    ABRY                 5
 -- NSW   2830  YARRABAR BARY    ABRY                 6 
 -- 7 row(s)

*********************************************************************************


